"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const fs = require("fs");
const path_1 = require("path");
const webpack_config_1 = require("./webpack-config");
const config_1 = require("./config");
const chalk_1 = require("chalk");
runCompiler();
function runCompiler() {
    return __awaiter(this, void 0, void 0, function* () {
        const rootDirectory = process.cwd();
        const config = config_1.getConfig();
        const clientConfig = webpack_config_1.genWebpackConfig({
            isDev: false, isServer: false, rootDirectory,
        });
        const serverConfig = webpack_config_1.genWebpackConfig({
            isDev: false, isServer: true, rootDirectory,
        });
        rmdir(config.buildDirectory);
        rmdir(config.staticDirectory);
        console.log(chalk_1.default.green('打包服务端资源文件...'));
        yield compiler(serverConfig);
        console.log(chalk_1.default.green('打包客户端资源文件...'));
        yield compiler(clientConfig);
        console.log(chalk_1.default.green('资源打包完成!'));
    });
}
function compiler(config) {
    return new Promise((resolve, reject) => {
        webpack(config).run((err, stats) => {
            if (err)
                reject(err);
            const { warnings, errors } = stats.toJson();
            if (warnings.length) {
                warnings.forEach(warning => {
                    console.log(chalk_1.default.yellowBright(warning));
                });
            }
            if (errors.length) {
                errors.forEach(error => {
                    console.log(chalk_1.default.red(error));
                });
                process.exit(1);
            }
            resolve();
        });
    });
}
function rmdir(directory) {
    if (fs.existsSync(directory)) {
        const files = fs.readdirSync(directory);
        if (files.length) {
            files.forEach(filename => {
                const filePath = path_1.join(directory, filename);
                const stats = fs.statSync(filePath);
                if (stats.isDirectory()) {
                    rmdir(filePath);
                }
                else {
                    fs.unlinkSync(filePath);
                }
            });
        }
        fs.rmdirSync(directory);
    }
}
