'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var Koa = require('koa');
var KoaRouter = require('koa-router');
var cheerio = require('cheerio');
var fs = require('fs');
var url = require('url');
var server = require('react-dom/server');
var H = require('history');
var path = require('path');
var path2Regexp = _interopDefault(require('path-to-regexp'));
var send = _interopDefault(require('koa-send'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const routerStore = {
    location: {
        pathname: '',
        search: '',
        state: null,
        hash: '',
    },
};
const RouterContext = React.createContext(routerStore);

class History {
    constructor() {
        this.subscriptions = [];
        this.push = (path, state) => {
            this.history.push(path, state);
        };
        this.replace = (path, state) => {
            this.history.replace(path, state);
        };
        this.go = (n) => {
            this.history.go(n);
        };
        this.goBack = () => {
            this.history.goBack();
        };
        this.subscribe = (subscriber) => {
            this.subscriptions.push(subscriber);
        };
        this.handleHistoryChange = (location) => {
            const subscriptions = this.subscriptions;
            subscriptions.forEach(subscriber => subscriber(location));
        };
        const history = 'undefined' === typeof window
            ? H.createMemoryHistory()
            : H.createBrowserHistory();
        history.listen(this.handleHistoryChange);
        this.history = history;
    }
}
const history = new History();

const DEFAULT_WEBPACK_SERVER_PORT = 8080;
const DEFAULT_DEV_SERVER_PORT = 3030;
const DEFAULT_CLIENT_PUBLIC_PATH = '/static/';
const DEFAULT_SERVER_PUBLIC_PATH = '/';
const rootDirectory = process.cwd();
function getConfig() {
    const htmlFilename = 'client.html';
    const staticDirName = 'static';
    const buildDirName = 'build';
    const srcDirectory = path.join(rootDirectory, 'src');
    return {
        publicFilesDirectory: path.join(rootDirectory, 'public'),
        htmlFilename,
        buildDirName,
        staticDirName,
        clientChunkName: 'app.js',
        serverChunkName: 'server.js',
        webpackServerPort: DEFAULT_WEBPACK_SERVER_PORT,
        serverPort: DEFAULT_DEV_SERVER_PORT,
        clientPublicPath: DEFAULT_CLIENT_PUBLIC_PATH,
        serverPublicPath: DEFAULT_SERVER_PUBLIC_PATH,
        htmlTemplatePath: path.join(srcDirectory, 'index.html'),
        htmlPath: path.join(rootDirectory, staticDirName, 'client.html'),
        buildDirectory: path.join(rootDirectory, buildDirName),
        staticDirectory: path.join(rootDirectory, staticDirName),
        srcDirectory,
        customConfigFile: path.join(rootDirectory, 'server-renderer.config.js'),
    };
}

class DefaultApp extends React.Component {
    static getInitialProps(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (params.Component && params.Component.getInitialProps) {
                const data = yield params.Component.getInitialProps(params);
                return {
                    data,
                };
            }
            return {
                data: {},
            };
        });
    }
    render() {
        const { Component, data } = this.props;
        return (React.createElement(Component, Object.assign({}, data)));
    }
}

class Router extends React.Component {
    constructor(props) {
        super(props);
        this.handleHistoryChange = (location) => {
            this.setState({ location });
        };
        const url$1 = url.parse(props.location);
        this.state = {
            location: {
                state: null,
                pathname: url$1.pathname,
                search: url$1.search,
                hash: url$1.hash,
            },
        };
        history.subscribe(this.handleHistoryChange);
    }
    shouldComponentUpdate(nextProps, nextState) {
        return this.state.location !== nextState.location;
    }
    render() {
        return (React.createElement(RouterContext.Provider, { value: this.state }, this.props.children));
    }
}

class Container extends React.PureComponent {
    constructor(props) {
        super(props);
        this.fetchInitialProps = (matchedRoute, url) => __awaiter(this, void 0, void 0, function* () {
            if (!matchedRoute) {
                return null;
            }
            const { App } = this.props;
            if (App && App.getInitialProps) {
                const initialProps = yield App.getInitialProps({
                    url,
                    Component: matchedRoute.component,
                });
                this.setState({ pageProps: initialProps });
            }
        });
        this.handleHistoryChange = (location) => {
            const { routes } = this.props;
            const matchedRoute = this.getMatchedRoute(routes, location.pathname);
            this.setState({
                component: matchedRoute ? matchedRoute.component : null,
                pageProps: {},
            });
            this.fetchInitialProps(matchedRoute, window.location.toString());
        };
        const { routes, pageProps, location } = props;
        const matchedRoute = this.getMatchedRoute(routes, url.parse(location).pathname);
        const component = matchedRoute && matchedRoute.component;
        this.state = {
            component,
            pageProps,
        };
        history.subscribe(this.handleHistoryChange);
    }
    render() {
        const { App, Error } = this.props;
        const { component, pageProps } = this.state;
        if (!component) {
            return (React.createElement(Error, { error: 'Page not found' }));
        }
        return (React.createElement(App, Object.assign({ Component: component }, pageProps)));
    }
    getMatchedRoute(routes, pathname) {
        const matched = routes.find(route => {
            return path2Regexp(route.path, [], { strict: true }).test(pathname);
        });
        return matched || routes.find(route => route.path === '*' || !route.path);
    }
}

const Link = (_a) => {
    var { className, activeClassName, to, onClick } = _a, restProps = __rest(_a, ["className", "activeClassName", "to", "onClick"]);
    const { location } = React.useContext(RouterContext);
    const matched = path2Regexp(location.pathname).test(to);
    const cls = className + matched ? activeClassName : '';
    const handleClick = (evt) => {
        if (onClick) {
            onClick(evt);
        }
        if (!evt.isDefaultPrevented()) {
            evt.preventDefault();
            history.push(to);
        }
    };
    return (React.createElement("a", Object.assign({ href: to, className: cls, onClick: handleClick }, restProps)));
};
Link.defaultProps = {
    className: '',
    activeClassName: '',
};

const Error = ({ error }) => {
    return (React.createElement("div", { "data-error": true }, String(error)));
};

const config = getConfig();
const isDev = process.env.NODE_ENV === 'development';
class Server {
    constructor(opts) {
        this.clientChunkPath = new url.URL(config.clientChunkName, `http://localhost:${config.webpackServerPort}${config.clientPublicPath}`);
        this.container = opts.container;
        this.App = opts.App || DefaultApp;
        this.Error = opts.Error || Error;
        this.routes = opts.routes;
        const htmlPath = isDev ? config.htmlTemplatePath : config.htmlPath;
        this.originalHTML = fs.readFileSync(htmlPath, 'utf-8');
    }
    start() {
        const app = new Koa();
        const router = new KoaRouter();
        const port = config.serverPort;
        app.use(this.serveFiles);
        router.get('*', this.handleRequest.bind(this));
        app.use(router.routes());
        app.listen(port, () => {
            console.log('Server listen on: http://localhost:' + port);
        });
    }
    handleRequest(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const routes = this.routes;
            const url$1 = ctx.url;
            const fullUrl = ctx.origin + url$1;
            const matchedRoute = routes.find(route => {
                const p = url.parse(url$1).pathname;
                return path2Regexp(route.path, [], { strict: true }).test(p);
            });
            const App = this.App;
            const Error = this.Error;
            const pageProps = yield this.getInitialProps(ctx, App, matchedRoute, fullUrl);
            const content = server.renderToString(React.createElement(Router, { location: fullUrl },
                React.createElement(Container, { location: fullUrl, App: App, Error: Error, pageProps: pageProps, routes: routes })));
            ctx.body = this.renderHTML(content, pageProps);
        });
    }
    serveFiles(ctx, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const staticDirName = config.staticDirName;
            const publicFilesDirectory = isDev
                ? config.buildDirectory
                : config.staticDirectory;
            const path = ctx.path;
            if (path.startsWith(`/${staticDirName}/`)) {
                yield send(ctx, path.replace(`/${staticDirName}/`, ''), { root: publicFilesDirectory });
            }
            else {
                yield next();
            }
        });
    }
    renderHTML(content, pageProps) {
        const $ = cheerio.load(this.originalHTML, { decodeEntities: true });
        $(this.container).html(content);
        $('head').append(`
      <script type='text/javascript'>
          __APP_DATA__="${encodeURIComponent(JSON.stringify({ pageProps }))}"
      </script>
    `);
        if (isDev) {
            $('body').append(`
        <script type='text/javascript' src='${this.clientChunkPath}'></script>
      `);
        }
        return $.html();
    }
    getInitialProps(ctx, AppContainer, matchedRoute, url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (matchedRoute && AppContainer.getInitialProps) {
                return yield AppContainer.getInitialProps({
                    Component: matchedRoute.component,
                    url,
                    ctx,
                });
            }
            return {};
        });
    }
}
function render(opts) {
    const server = new Server(opts);
    server.start();
}

Object.keys(H).forEach(function (key) {
    Object.defineProperty(exports, key, {
        enumerable: true,
        get: function () {
            return H[key];
        }
    });
});
exports.Link = Link;
exports.RouterContext = RouterContext;
exports.history = history;
exports.render = render;
